#include <iostream>
#include <vector>
#include <string>
#include <algorithm>
#include <type_traits>

#include <local/vector.hpp>


template <class vector_t>
void test_vector_display(vector_t const & v)
{
	std::cout << "Display with for loop       = ";
	for (size_t i = 0; i < v.size(); ++i)
	{
		std::cout << v[i] << " ";
	}
	std::cout << std::endl;
	
	std::cout << "Display with for range loop = " ;
	for (auto const & e : v)
	{
		std::cout << e << " ";
	}
	std::cout << std::endl;
}

template <class vector_t>
void test_vector_sort(vector_t & v)
{
	std::cout << "Sorted =" << std::endl;
	std::sort(v.begin(), v.end());
}

template <class vector_t>
typename vector_t::value_type test_vector_sum(vector_t const & v)
{
	using T = typename vector_t::value_type;
	return std::accumulate(v.begin(), v.end(), T());
}

template <class vector_t>
typename vector_t::value_type test_vector_average(vector_t const & v)
{
	using T = typename vector_t::value_type;
	
	return test_vector_sum(v) / T(v.size());
}

template <class vector_t>
void test_vector(std::false_type const, vector_t & v)
{
	test_vector_display(v);
	std::cout << std::endl;
	
	test_vector_sort(v);
	test_vector_display(v);
	std::cout << std::endl;
	
	std::cout << "Sum     = " << test_vector_sum(v) << std::endl;
}

template <class vector_t>
void test_vector(std::true_type const, vector_t & v)
{
	test_vector(std::false_type(), v);
	std::cout << "Average = " << test_vector_average(v) << std::endl;
}

template <class vector_t>
void test_vector(vector_t & v)
{
	using T = typename vector_t::value_type;
	
	auto copy = v;
	
	test_vector(std::is_arithmetic<T>(), copy);
}


int main()
{
	std::cout << std::endl;
	
	// std::vector
	
	std::cout << " ------------------" << std::endl;
	std::cout << "| std::vector<int> |" << std::endl;
	std::cout << " ------------------" << std::endl;
	std::cout << std::endl;
	{
		std::vector<int> v({ 8, 5, 1, 3, 2 });
		test_vector(v);
	}
	std::cout << std::endl;
	
	std::cout << " ---------------------" << std::endl;
	std::cout << "| std::vector<double> |" << std::endl;
	std::cout << " ---------------------" << std::endl;
	std::cout << std::endl;
	{
		std::vector<double> v({ 8.0, 5.0, 1.0, 3.0, 2.0 });
		test_vector(v);
	}
	std::cout << std::endl;
	
	std::cout << " --------------------------" << std::endl;
	std::cout << "| std::vector<std::string> |" << std::endl;
	std::cout << " --------------------------" << std::endl;
	std::cout << std::endl;
	{
		std::vector<std::string> v({ "A", "vector", "of", "std::string" });
		test_vector(v);
	}
	std::cout << std::endl;
	
	// local::vector
	/*
	std::cout << " ----------------------" << std::endl;
	std::cout << "| local::vector<float> |" << std::endl;
	std::cout << " ----------------------" << std::endl;
	std::cout << std::endl;
	{
		local::vector<float> v({ 73.f, 21.0f, 42.0f });
		test_vector(v);
	}
	std::cout << std::endl;
	
	std::cout << " ----------------------------" << std::endl;
	std::cout << "| local::vector<std::string> |" << std::endl;
	std::cout << " ----------------------------" << std::endl;
	std::cout << std::endl;
	{
		local::vector<std::string> v({ "A", "vector", "of", "local::string" });
		test_vector(v);
	}
	std::cout << std::endl;
	*/
	return 0;
}
